# -*- coding: utf-8 -*-

from datetime import datetime
from freezegun import freeze_time
from imio.events.core.rest.endpoint import EventsEndpointHandler
from imio.events.core.testing import IMIO_EVENTS_CORE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.memoize.interfaces import ICacheChooser
from plone import api
from unittest.mock import patch
from zope.component import getUtility

import unittest


def clear_search_cache(query):
    cache_key = EventsEndpointHandler._cache_key("func", "instance", query)
    cache = getUtility(ICacheChooser)(cache_key)
    storage = cache.ramcache._getStorage()._data
    del storage["imio.events.core.rest.endpoint.search"]


class RestFunctionalTest(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity",
        )
        self.agenda = api.content.create(
            container=self.entity,
            type="imio.events.Agenda",
            id="agenda",
        )

    def test_query(self):
        endpoint = EventsEndpointHandler(self.portal, self.request)
        # None query
        query = None
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 0)
        clear_search_cache(query)

    @freeze_time("2023-03-18")
    def test_get_events_from_agenda(self):
        event1 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event1",
            title="Event 1",
        )
        event1.start = datetime(2023, 3, 27, 0, 0)
        event1.end = datetime(2023, 4, 27, 0, 0)
        event1.reindexObject()

        event2 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event2",
            title="Event 2",
        )
        event2.start = datetime(2023, 3, 18, 0, 0)
        event2.end = datetime(2023, 3, 18, 23, 59)
        event2_nb_occurences = 3
        event2.recurrence = f"RRULE:FREQ=WEEKLY;COUNT={event2_nb_occurences}"
        event2.reindexObject()

        event3 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event3",
            title="Event 3",
        )
        event3.start = datetime(2023, 3, 25, 0, 0)
        event3.end = datetime(2023, 3, 25, 23, 59)
        endpoint = EventsEndpointHandler(self.portal, self.request)
        query = {
            "b_size": 10,
            "b_start": 0,
        }

        # start in past , end in future
        event4 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event4",
            title="Event 4",
        )
        event4.start = datetime(2023, 3, 1, 0, 0)
        event4.end = datetime(2023, 4, 27, 0, 0)
        event4.reindexObject()

        # All tests below are done to retrieve FUTURE events
        self.request.form["event_dates.range"] = "min"

        response = endpoint.search(query)
        items = response.get("items")
        # Assert private event are not presents
        self.assertEqual(len(items), 0)
        clear_search_cache(query)

        # Assert published event are presents
        api.content.transition(event1, "publish")
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 1)
        clear_search_cache(query)

        # Assert event that began in past and end in future are presents
        api.content.transition(event4, "publish")
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 2)
        api.content.transition(event4, "retract")
        clear_search_cache(query)

        # Assert events occurrences are presents
        api.content.transition(event2, "publish")
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 1 + event2_nb_occurences)
        clear_search_cache(query)

        # Assert passed event aren't get anymore (start_date and end_date are in the past)
        # But future occurrences of this event are kept
        event2.start = datetime(2023, 3, 16, 0, 0)
        event2.reindexObject()
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 3)
        clear_search_cache(query)

        event4.start = datetime(2023, 3, 1, 0, 0)
        event4.end = datetime(2023, 3, 29, 0, 0)
        event4.recurrence = "RRULE:FREQ=WEEKLY;COUNT=5"
        api.content.transition(event4, "publish")
        event4.reindexObject()
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 5)
        api.content.transition(event4, "retract")
        event4.reindexObject()
        clear_search_cache(query)

        # Assert first_start index was updated
        item = list(
            set([i.get("first_start") for i in items if i.get("title") == "Event 2"])
        )
        self.assertEqual(item[0], "2023-03-16T00:00:00")

        api.content.transition(event3, "publish")
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 4)
        clear_search_cache(query)

        # Assert list is sorted by start date
        format_str = "%Y-%m-%dT%H:%M:%S%z"
        dates = [datetime.strptime(i.get("start"), format_str) for i in items]
        self.assertEqual(min(dates), dates[0])
        self.assertEqual(max(dates), dates[-1])

    @freeze_time("2023-03-18")
    @patch("requests.post")
    def test_batching_events_from_agenda(self, m):
        event1 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event1",
            title="Event 1",
        )
        event1.start = datetime(2023, 3, 27, 0, 0)
        event1.end = datetime(2023, 4, 27, 0, 0)
        event1.reindexObject()

        event2 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event2",
            title="Event 2",
        )
        event2.start = datetime(2023, 3, 18, 0, 0)
        event2.end = datetime(2023, 3, 18, 23, 59)
        event2_nb_occurences = 3
        event2.recurrence = f"RRULE:FREQ=WEEKLY;COUNT={event2_nb_occurences}"
        event2.reindexObject()

        event3 = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            id="event3",
            title="Event 3",
        )
        event3.start = datetime(2023, 3, 25, 0, 0)
        event3.end = datetime(2023, 3, 25, 23, 59)
        event3.reindexObject()
        endpoint = EventsEndpointHandler(self.portal, self.request)
        api.content.transition(event1, "publish")
        api.content.transition(event2, "publish")
        api.content.transition(event3, "publish")

        query = {
            "b_size": 10,
            "b_start": 0,
        }
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 5)
        clear_search_cache(query)

        query = {
            "b_size": 3,
            "b_start": 0,
        }
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 3)
        clear_search_cache(query)

        query = {
            "b_size": 3,
            "b_start": 3,
        }
        response = endpoint.search(query)
        items = response.get("items")
        self.assertEqual(len(items), 2)
        clear_search_cache(query)



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timedelta
from eea.facetednavigation.settings.interfaces import IHidePloneLeftColumn
from imio.events.core.contents import IAgenda
from imio.events.core.contents import IEntity
from imio.smartweb.common.faceted.utils import configure_faceted
from plone import api
from plone.event.recurrence import recurrence_sequence_ical
from plone.restapi.serializer.converters import json_compatible
from Products.CMFPlone.utils import parent
from pytz import utc
from zope.component import getMultiAdapter
from zope.interface import noLongerProvides

import copy
import dateutil
import os


def get_entity_for_obj(obj):
    while not IEntity.providedBy(obj) and obj is not None:
        obj = parent(obj)
    entity = obj
    return entity


def get_agenda_for_event(event):
    obj = event
    while not IAgenda.providedBy(obj) and obj is not None:
        obj = parent(obj)
    agenda = obj
    return agenda


def get_agendas_uids_for_faceted(obj):
    if IAgenda.providedBy(obj):
        return [obj.UID()]
    elif IEntity.providedBy(obj):
        brains = api.content.find(context=obj, portal_type="imio.events.Agenda")
        return [b.UID for b in brains]
    else:
        raise NotImplementedError


def reload_faceted_config(obj, request):
    faceted_config_path = "{}/faceted/config/events.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)
    agendas_uids = "\n".join(get_agendas_uids_for_faceted(obj))
    request.form = {
        "cid": "agenda",
        "faceted.agenda.default": agendas_uids,
    }
    handler = getMultiAdapter((obj, request), name="faceted_update_criterion")
    handler.edit(**request.form)
    if IHidePloneLeftColumn.providedBy(obj):
        noLongerProvides(obj, IHidePloneLeftColumn)


def get_start_date(event):
    return datetime.fromisoformat(event["start"])


# just expand occurences. No filtering here
def expand_occurences(events, range="min"):
    expanded_events = []

    for event in events:
        start_date = dateutil.parser.parse(event["first_start"])
        start_date = start_date.astimezone(utc)
        end_date = dateutil.parser.parse(event["first_end"])
        end_date = end_date.astimezone(utc)

        # Ensure event start/end are in same date format than other json dates
        event["start"] = json_compatible(start_date)
        event["end"] = json_compatible(end_date)

        if not event["recurrence"]:
            expanded_events.append(event)
            continue

        # optimize query with "until" to avoid to go through all recurrences
        # if we want "future events", we get occurences to 5 years in the future
        # if we want "past events", we get occurences to 1 year in the past
        until = from_ = None
        until = start_date + timedelta(days=365 * 5)
        # for now min:max is only supported for future events
        if range == "min:max":
            from_ = datetime.now()
        elif range == "max":
            from_ = start_date - timedelta(days=365)
            until = datetime.now()
        start_dates = recurrence_sequence_ical(
            start=start_date,
            recrule=event["recurrence"],
            from_=from_,
            until=until,
        )

        if event["whole_day"] or event["open_end"]:
            duration = timedelta(hours=23, minutes=59, seconds=59)
        else:
            duration = end_date - start_date

        for occurence_start in start_dates:
            new_event = copy.deepcopy(event)
            start_time = datetime.combine(datetime.today(), start_date.time())
            end_time = datetime.combine(datetime.today(), end_date.time())
            duration = end_time - start_time
            new_event["start"] = json_compatible(occurence_start)
            new_event["end"] = json_compatible(occurence_start + duration)
            expanded_events.append(new_event)
    return expanded_events


def remove_zero_interval_from_recrule(recrule):
    if not recrule:
        return recrule
    recrule = recrule.replace(";INTERVAL=0", "")
    return recrule



# -*- coding: utf-8 -*-

from imio.events.core.testing import IMIO_EVENTS_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import unittest


class TestVocabularies(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_event_categories(self):
        factory = getUtility(
            IVocabularyFactory, "imio.events.vocabulary.EventsCategories"
        )
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 10)

    def test_events_local_categories_on_root(self):
        factory = getUtility(
            IVocabularyFactory, "imio.events.vocabulary.EventsLocalCategories"
        )
        vocabulary = factory(self.portal)
        self.assertEqual(len(vocabulary), 0)

    def test_event_categories_topics(self):
        entity = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="imio.events.Entity",
            local_categories=[],
        )

        factory = getUtility(
            IVocabularyFactory,
            "imio.events.vocabulary.EventsCategoriesAndTopicsVocabulary",
        )
        vocabulary = factory(entity)
        self.assertEqual(len(vocabulary), 27)  # must be updated if add new vocabulary

    def test_event_categories_topics_local_cat(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        entity = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="imio.events.Entity",
            local_categories=[
                {"fr": "Foo", "nl": "", "de": "", "en": ""},
                {"fr": "baz", "nl": "", "de": "", "en": ""},
                {"fr": "bar", "nl": "", "de": "", "en": ""},
            ],
        )
        agenda = api.content.create(
            container=entity,
            type="imio.events.Agenda",
            id="imio.events.Agenda",
        )
        event_item = api.content.create(
            container=agenda,
            type="imio.events.Event",
            id="imio.events.Event",
        )

        factory = getUtility(
            IVocabularyFactory,
            "imio.events.vocabulary.EventsCategoriesAndTopicsVocabulary",
        )
        vocabulary = factory(event_item)
        self.assertEqual(len(vocabulary), 30)  # must be updated if add new vocabulary

    def test_agendas_UIDs(self):
        entity1 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            title="Entity1",
        )
        entity2 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            title="Entity2",
        )
        agenda1 = api.content.create(
            container=entity1,
            type="imio.events.Agenda",
            title="Agenda1",
        )
        agenda2 = api.content.create(
            container=entity2,
            type="imio.events.Agenda",
            title="Agenda2",
        )
        folder = api.content.create(
            container=agenda1,
            type="imio.events.Folder",
            title="Folder",
        )
        event1 = api.content.create(
            container=folder,
            type="imio.events.Event",
            title="Event1",
        )
        event2 = api.content.create(
            container=agenda2,
            type="imio.events.Event",
            title="Event2",
        )

        all_agendas = []
        ag_entity1 = entity1.listFolderContents(
            contentFilter={"portal_type": "imio.events.Agenda"}
        )
        ag_entity2 = entity2.listFolderContents(
            contentFilter={"portal_type": "imio.events.Agenda"}
        )
        all_agendas = [*set(ag_entity1 + ag_entity2)]

        factory = getUtility(IVocabularyFactory, "imio.events.vocabulary.AgendasUIDs")
        vocabulary = factory(self.portal)
        self.assertEqual(len(vocabulary), len(all_agendas))

        vocabulary = factory(event1)
        self.assertEqual(len(vocabulary), len(all_agendas))

        vocabulary = factory(event2)
        uid = agenda2.UID()
        vocabulary.getTerm(uid)
        self.assertEqual(vocabulary.getTerm(uid).title, "Entity2 » Agenda2")

        vocabulary = factory(self.portal)
        ordered_agendas = [a.title for a in vocabulary]
        titles = []
        for agenda in ag_entity1 + ag_entity2:
            titles.append(f"{agenda.aq_parent.Title()} » {agenda.Title()}")
        titles.sort()
        ordered_agendas.sort()
        self.assertEqual(ordered_agendas, titles)
        agenda1.title = "Z Change order!"
        agenda1.reindexObject()
        vocabulary = factory(self.portal)
        ordered_agendas = [a.title for a in vocabulary]
        # "Entity2 » Agenda2", "Z Change order! » Agenda1"
        self.assertIn("Entity1 » Z Change order!", ordered_agendas)

    def test_event_types(self):
        factory = getUtility(IVocabularyFactory, "imio.events.vocabulary.EventTypes")
        vocabulary = factory(self.portal)
        self.assertEqual(len(vocabulary), 2)



# -*- coding: utf-8 -*-

from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
)
from plone.api import portal as portal_api
from plone.testing import z2
from zope.globalrequest import setRequest

import imio.events.core
import mock


class ImioEventsCoreLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=imio.events.core)
        self.loadZCML(package=imio.events.core, name="overrides.zcml")

    def setUpPloneSite(self, portal):
        request = portal.REQUEST
        # set basic request to be able to get current language from it during
        # policy setuphandlers
        setRequest(request)
        portal_api.get_current_language = mock.Mock(return_value="fr")
        applyProfile(portal, "imio.events.core:default")


IMIO_EVENTS_CORE_FIXTURE = ImioEventsCoreLayer()


IMIO_EVENTS_CORE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(IMIO_EVENTS_CORE_FIXTURE,),
    name="ImioEventsCoreLayer:IntegrationTesting",
)


IMIO_EVENTS_CORE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(IMIO_EVENTS_CORE_FIXTURE,),
    name="ImioEventsCoreLayer:FunctionalTesting",
)


IMIO_EVENTS_CORE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        IMIO_EVENTS_CORE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="ImioEventsCoreLayer:AcceptanceTesting",
)



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timedelta
from datetime import timezone
from imio.events.core.utils import expand_occurences
from imio.events.core.utils import get_start_date
from imio.smartweb.common.rest.search_filters import SearchFiltersHandler
from imio.smartweb.common.utils import is_log_active
from plone import api
from plone.memoize import ram
from plone.restapi.batching import HypermediaBatch
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from zope.component import getMultiAdapter

import logging
import time

logger = logging.getLogger("imio.events.core")


class SearchFiltersGet(Service):
    """
    This is a temporary shortcut to calculate search-filters based on @events
    search results logic. We need to refactor & test (more) this module.
    """

    def reply(self):
        query = self.request.form.copy()
        if "metadata_fields" not in query:
            return {}
        query = unflatten_dotted_dict(query)
        results = EventsEndpointHandler(self.context, self.request).search(query)
        UIDs = [item["UID"] for item in results["items"]]
        query = {
            "UID": UIDs,
            "metadata_fields": ["category", "local_category", "topics"],
        }
        return SearchFiltersHandler(self.context, self.request).search(query)


class EventsEndpointGet(Service):
    def reply(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        return EventsEndpointHandler(self.context, self.request).search(query)


class EventsEndpointHandler(SearchHandler):
    """ """

    # we receive b_size and b_start from smartweb with values already set
    # So we ignore these values but we must stock these to use it ...
    ignored_params = ["b_size", "b_start"]

    def _cache_key(func, instance, query):
        return (query, time.time() // 60)

    @ram.cache(_cache_key)
    def search(self, query=None):
        if query is None:
            query = {}
        b_size = query.get("b_size") or 20
        b_start = query.get("b_start") or 0

        for param in self.ignored_params:
            if param in query:
                del query[param]

        if "fullobjects" in query:
            fullobjects = True
            del query["fullobjects"]
        else:
            fullobjects = False

        query["portal_type"] = "imio.events.Event"
        query["review_state"] = "published"
        query["b_size"] = 10000

        def cascading_agendas(initial_agenda):
            global_list = []

            def recursive_generator(agenda_UID):
                nonlocal global_list
                obj = api.content.get(UID=agenda_UID)
                populating_agendas = []
                for rv in obj.populating_agendas:
                    if hasattr(rv, "to_object"):
                        if (
                            rv.to_object is not None
                            and rv.to_object.UID() not in global_list
                        ):
                            obj = rv.to_object
                            status = api.content.get_state(obj)
                            if status == "published":
                                # to cover initial_agenda UID
                                populating_agendas.append(agenda_UID)
                                global_list.append(agenda_UID)

                                # to cover RelationValue agenda UID
                                populating_agendas.append(rv.to_object.UID())
                                global_list.append(rv.to_object.UID())
                for agenda_UID in populating_agendas:
                    yield from recursive_generator(agenda_UID)
                yield agenda_UID

            yield from recursive_generator(initial_agenda)

        # To cover cascading "populating_agendas" field
        if "selected_agendas" in query:
            selected_agendas = [
                agenda_UID
                for agenda_UID in cascading_agendas(query["selected_agendas"])
            ]
            all_agendas = list(set(selected_agendas))
            query["selected_agendas"] = all_agendas
        tps1 = time.time()
        self._constrain_query_by_path(query)
        # self.request.form["path"] = query["path"]
        tps2 = time.time()
        query = self._parse_query(query)
        tps3 = time.time()
        range = self.request.form.get("event_dates.range")

        # Try to optimize the query
        if range == "max":
            query["b_size"] = 1000
            now = datetime.now(timezone.utc)
            one_year_ago = now - timedelta(days=365)
            date_string = now.strftime("%Y-%m-%d")
            one_year_ago_string = one_year_ago.strftime("%Y-%m-%d")
            query["event_dates"]["query"] = [one_year_ago_string, date_string]
            query["event_dates"]["range"] = "min:max"
            self.request.form["event_dates.query"] = {
                "min": one_year_ago_string,
                "max": date_string,
            }
            self.request.form["event_dates.range"] = "min:max"
        lazy_resultset = self.catalog.searchResults(**query)
        tps4 = time.time()
        if "metadata_fields" not in self.request.form:
            self.request.form["metadata_fields"] = []
        self.request.form["metadata_fields"] += [
            "container_uid",
            "recurrence",
            "whole_day",
            "first_start",
            "first_end",
            "open_end",
        ]
        # ISerializeToJson use a default request batch so we force a "full" b_size and a "zero" b_start
        self.request.form["b_size"] = 10000
        self.request.form["b_start"] = 0

        # To do : cache all datas and get from cache ?
        results = getMultiAdapter((lazy_resultset, self.request), ISerializeToJson)(
            fullobjects=fullobjects
        )
        tps5 = time.time()
        expanded_occurences = expand_occurences(results.get("items"), range)
        tps6 = time.time()
        # range is None when we click on an occurence
        if range is None:
            sorted_expanded_occurences = expanded_occurences
        if range == "min":
            filter_expanded_occurences = []
            for occurrence in expanded_occurences:
                start_date = datetime.fromisoformat(occurrence["start"])
                end_date = datetime.fromisoformat(occurrence["end"])
                current_date = datetime.now(timezone.utc)
                if start_date >= current_date or end_date >= current_date:
                    filter_expanded_occurences.append(occurrence)
            sorted_expanded_occurences = sorted(
                filter_expanded_occurences, key=get_start_date
            )
        if range == "max":
            filter_expanded_occurences = []
            for occurrence in expanded_occurences:
                end_date = datetime.fromisoformat(occurrence["end"])
                current_date = datetime.now(timezone.utc)
                if end_date < current_date:
                    filter_expanded_occurences.append(occurrence)
            sorted_expanded_occurences = sorted(
                filter_expanded_occurences, key=get_start_date, reverse=True
            )
        if range == "min:max":
            filter_expanded_occurences = []
            for occurrence in expanded_occurences:
                min_date, max_date = self.request.form.get("event_dates.query")
                min_date = datetime.fromisoformat(min_date).replace(tzinfo=timezone.utc)
                max_date = (
                    datetime.strptime(max_date, "%Y-%m-%d")
                    .replace(hour=23, minute=59, second=59)
                    .replace(tzinfo=timezone.utc)
                )
                start_date = datetime.fromisoformat(occurrence["start"])
                end_date = datetime.fromisoformat(occurrence["end"])
                if (min_date <= start_date <= max_date) or (
                    start_date <= min_date and end_date >= min_date
                ):
                    filter_expanded_occurences.append(occurrence)
            sorted_expanded_occurences = sorted(
                filter_expanded_occurences, key=get_start_date
            )
        tps7 = time.time()
        # It's time to get real b_size/b_start from the smartweb query
        self.request.form["b_size"] = b_size
        self.request.form["b_start"] = b_start
        batch = HypermediaBatch(self.request, sorted_expanded_occurences)
        if is_log_active():
            logger.info(f"query : {results['@id']}")
            logger.info(f"time constrain_query_by_path : {tps2 - tps1}")
            logger.info(f"time _parse_query : {tps3 - tps2}")
            logger.info(f"time catalog lazy_resultset : {tps4 - tps3}")
            logger.info(f"time MultiAdapter fullobj : {tps5 - tps4}")
            logger.info(f"time occurences : {tps6 - tps5}")
            logger.info(f"time batch : {tps7 - tps6}")
            logger.info(f"time (total) : {tps7 - tps1}")

        results = {}
        results["@id"] = batch.canonical_url
        results["items_total"] = batch.items_total
        links = batch.links
        if links:
            results["batching"] = links
        results["items"] = [event for event in batch]
        return results



# -*- coding: utf-8 -*-

from imio.events.core.contents import IAgenda
from imio.events.core.contents import IEvent
from imio.events.core.contents import IFolder
from imio.events.core.interfaces import IImioEventsCoreLayer
from imio.smartweb.common.rest.utils import get_restapi_query_lang
from imio.smartweb.common.utils import is_log_active
from plone import api
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.restapi.serializer.summary import DefaultJSONSummarySerializer
from Products.CMFCore.WorkflowCore import WorkflowException
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory

import logging

logger = logging.getLogger("imio.events.core")


def get_container_uid(event_uid, summary=None):
    if summary is not None:
        container_uid = summary.get("UID") or summary.get("container_uid")
        if container_uid:
            return container_uid
    if event_uid is None:
        return None
    brain = api.content.find(UID=event_uid)[0]
    container_uid = getattr(brain, "container_uid", None)
    return container_uid


@implementer(ISerializeToJson)
@adapter(IEvent, IImioEventsCoreLayer)
class SerializeEventToJson(SerializeFolderToJson):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeEventToJson, self).__call__(version, include_items=True)
        query = self.request.form
        lang = get_restapi_query_lang(query)
        result["first_start"] = json_compatible(self.context.start)
        result["first_end"] = json_compatible(self.context.end)
        title = result["title"]
        text = result["text"]
        desc = result["description"]

        if lang and lang != "fr":
            result["title"] = result[f"title_{lang}"]
            result["description"] = result[f"description_{lang}"]
            result["text"] = result[f"text_{lang}"]
            if self.context.local_category:
                factory = getUtility(
                    IVocabularyFactory, "imio.events.vocabulary.EventsLocalCategories"
                )
                vocabulary = factory(self.context, lang=lang)
                term = vocabulary.getTerm(self.context.local_category)
                result["local_category"] = {
                    "token": self.context.local_category,
                    "title": term.title,
                }

        # Getting agenda title/id to use it in rest views
        if query.get("metadata_fields") is not None and "container_uid" in query.get(
            "metadata_fields"
        ):
            agenda = None
            event_uid = self.context.UID()
            container_uid = get_container_uid(event_uid)
            if container_uid is not None:
                # Agendas can be private (admin to access them). That doesn't stop events to be bring.
                with api.env.adopt_user(username="admin"):
                    agenda = api.content.get(UID=container_uid)
                # To make a specific agenda css class in smartweb carousel common template
                result["usefull_container_id"] = agenda.id
                # To display agenda title in smartweb carousel common template
                result["usefull_container_title"] = agenda.title

        # maybe not necessary :
        result["title_fr"] = title
        result["description_fr"] = desc
        result["text_fr"] = text
        return result


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IImioEventsCoreLayer)
class EventJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self):
        summary = super(EventJSONSummarySerializer, self).__call__()
        query = self.request.form
        # To get agenda title and use it in carousel,...
        if query.get("metadata_fields") is not None and "container_uid" in query.get(
            "metadata_fields"
        ):
            agenda = None
            if IEvent.providedBy(self.context):
                event_uid = self.context.UID()
                container_uid = get_container_uid(event_uid)
            elif IAgenda.providedBy(self.context) or IFolder.providedBy(self.context):
                container_uid = get_container_uid(None, summary)
            elif IEvent.providedBy(self.context.getObject()):
                # context can be a brain
                event_uid = self.context.UID
                container_uid = get_container_uid(event_uid)
            else:
                container_uid = get_container_uid(None, summary)
            if container_uid is None:
                if is_log_active():
                    logger.info(f"container_uid is None ?")
                    logger.info(f"QUERY_STRING: {self.request.QUERY_STRING}")
                    logger.info(f"summary: {summary}")
                return summary
            # Agendas can be private (admin to access them). That doesn't stop events to be bring.
            with api.env.adopt_user(username="admin"):
                agenda = api.content.get(UID=container_uid)
            if not agenda:
                return summary
            # To make a specific agenda css class in smartweb carousel common template
            summary["usefull_container_id"] = agenda.id
            # To display agenda title in smartweb carousel common template
            summary["usefull_container_title"] = agenda.title
        lang = get_restapi_query_lang(query)
        if lang == "fr":
            # nothing to go, fr is the default language
            return summary

        obj = IContentListingObject(self.context)
        for orig_field in ["title", "description", "category_title", "local_category"]:
            field = f"{orig_field}_{lang}"
            accessor = self.field_accessors.get(field, field)
            value = getattr(obj, accessor, None) or None
            try:
                if callable(value):
                    value = value()
            except WorkflowException:
                summary[orig_field] = None
                continue
            if orig_field == "description" and value is not None:
                value = value.replace("**", "")
            summary[orig_field] = json_compatible(value)

        return summary



# -*- coding: utf-8 -*-

from datetime import datetime
from DateTime import DateTime
from imio.events.core.contents import IAgenda
from imio.events.core.contents import IEntity
from imio.events.core.contents import IEvent
from imio.events.core.utils import get_agenda_for_event
from imio.events.core.utils import get_entity_for_obj
from imio.smartweb.common.rest.odwb import OdwbBaseEndpointGet
from imio.smartweb.common.utils import is_log_active
from imio.smartweb.common.utils import (
    activate_sending_data_to_odwb_for_staging as odwb_staging,
)
from plone import api
from plone.formwidget.geolocation.geolocation import Geolocation
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot

import json
import logging
import requests

logger = logging.getLogger("imio.events.core")


class OdwbEndpointGet(OdwbBaseEndpointGet):
    def __init__(self, context, request):
        imio_service = (
            "evenements-en-wallonie"
            if not odwb_staging()
            else "staging-evenements-en-wallonie"
        )
        pushkey = f"imio.events.core.odwb_{imio_service}_pushkey"
        super(OdwbEndpointGet, self).__init__(context, request, imio_service, pushkey)

    def reply(self):
        if not super(OdwbEndpointGet, self).available():
            return
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/push/?pushkey={self.odwb_pushkey}"
        if is_log_active():
            logger.info(f"ODWB push url: {url}")
        self.__datas__ = self.get_events()
        batched_lst = [
            self.__datas__[i : i + 1000] for i in range(0, len(self.__datas__), 1000)
        ]
        for elem in batched_lst:
            payload = json.dumps(elem)
            response_text = self.odwb_query(url, payload)
            logger.info(response_text)
        return response_text

    def get_events(self):
        lst_events = []
        if IPloneSiteRoot.providedBy(self.context) or IAgenda.providedBy(self.context):
            brains = api.content.find(
                object_provides=IEvent.__identifier__, review_state="published"
            )
            for brain in brains:
                if IAgenda.providedBy(self.context):
                    if self.context.UID() not in brain.selected_agendas:
                        continue
                event_obj = brain.getObject()
                event = Event(event_obj)
                lst_events.append(json.loads(event.to_json()))
        elif IEvent.providedBy(self.context):
            event = Event(self.context)
            lst_events.append(json.loads(event.to_json()))
        return lst_events

    def remove(self):
        if not super(OdwbEndpointGet, self).available():
            return
        lst_events = []
        if IEvent.providedBy(self.context):
            event = Event(self.context)
            lst_events.append(json.loads(event.to_json()))
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/delete/?pushkey={self.odwb_pushkey}"
        if is_log_active():
            logger.info(f"ODWB delete url: {url}")
        payload = json.dumps(lst_events)
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", url, headers=headers, data=payload)
        return response.text


class Event:

    def __init__(self, context):
        # ODWB fields = imio.events.core fields
        self.id = context.id
        self.start_datetime = context.start
        self.end_datetime = context.end
        self.recurrence = context.recurrence
        self.whole_day = context.whole_day
        self.open_end = context.open_end
        self.title = context.title
        self.description = context.description
        self.image = f"{context.absolute_url()}/@@images/image/preview"
        self.category = context.category
        self.topics = context.topics
        self.target_audience = context.local_category
        self.attendees = context.attendees if hasattr(context, "attendees") else ()
        self.text = context.text.raw if context.text else None
        self.reduced_mobility_facilities = context.reduced_mobility_facilities
        self.free_entry = context.free_entry
        self.coordinates = context.geolocation
        self.address_street_name = context.street
        self.address_house_number = context.number
        self.address_postal_code = context.zipcode
        self.address_city = context.city
        self.address_country = context.country
        self.online_meeting_url = context.online_participation
        self.contact_name = context.contact_name
        self.contact_email = context.contact_email
        self.contact_phone = context.contact_phone
        self.ticket_url = context.ticket_url
        self.event_url = context.event_url
        self.facebook_url = context.facebook
        self.instagram_url = context.instagram
        self.twitter_url = context.twitter
        self.video_url = context.video_url
        self.owner_id = get_entity_for_obj(context).UID()
        self.owner_name = get_entity_for_obj(context).Title()
        self.owner_diary_id = get_agenda_for_event(context).UID()
        self.owner_diary_name = get_agenda_for_event(context).Title()
        self.creation_datetime = context.creation_date
        self.modification_datetime = context.modification_date

        # In a first time, We don't send these fields to ODWB
        # self.street_number_complement = context.complement
        # self.description_de = context.description_de
        # self.description_en = context.description_en
        # self.description_nl = context.description_nl
        # self.effective_date = context.effective_date
        # self.event_type = context.event_type  # event-driven,
        # self.exclude_from_nav = context.exclude_from_nav
        # self.expiration_date = context.expiration_date
        # self.latitude = context.geolocation.latitude if context.geolocation else None
        # self.longitude = context.geolocation.longitude if context.geolocation else None
        # self.iam = context.iam
        # self.language = context.language
        # self.local_category = context.local_category
        # self.subjects = context.subject
        # self.taxonomy_event_public = context.taxonomy_event_public
        # self.text_de = context.text_de.raw if context.text_de else None
        # self.text_en = context.text_en.raw if context.text_en else None
        # self.text_nl = context.text_nl.raw if context.text_nl else None
        # self.title_de = context.title_de
        # self.title_en = context.title_en
        # self.title_nl = context.title_nl

    def to_json(self):
        return json.dumps(self.__dict__, cls=EventEncoder)


class EventEncoder(json.JSONEncoder):

    def default(self, attr):
        if isinstance(attr, Geolocation):
            return {
                "lon": attr.longitude,
                "lat": attr.latitude,
            }
        elif isinstance(attr, DateTime):
            iso_datetime = attr.ISO8601()
            return iso_datetime
        elif isinstance(attr, datetime):
            return attr.isoformat()
        else:
            return super().default(attr)


class OdwbEntitiesEndpointGet(OdwbBaseEndpointGet):

    def __init__(self, context, request):
        imio_service = (
            "entites-des-agendas-en-wallonie"
            if not odwb_staging()
            else "staging-entites-des-agendas-en-wallonie"
        )
        pushkey = f"imio.events.core.odwb_{imio_service}_pushkey"
        super(OdwbEntitiesEndpointGet, self).__init__(
            context, request, imio_service, pushkey
        )

    def reply(self):
        if not super(OdwbEntitiesEndpointGet, self).available():
            return
        lst_entities = []
        brains = api.content.find(
            object_provides=IEntity.__identifier__, review_state="published"
        )
        for brain in brains:
            entity = {}
            entity["UID"] = brain.UID
            entity["id"] = brain.id
            entity["entity_title"] = brain.Title
            lst_entities.append(entity)
        self.__datas__ = lst_entities
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/push/?pushkey={self.odwb_pushkey}"
        if is_log_active():
            logger.info(f"ODWB push url: {url}")
        payload = json.dumps(lst_entities)
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", url, headers=headers, data=payload)
        return response.text



# -*- coding: utf-8 -*-

from imio.events.core.contents import IEntity
from plone import api
from plone.app.layout.viewlets import common


class BringEventIntoAgendasViewlet(common.ViewletBase):

    def available(self):
        form_name = "bring_event_into_agendas_form"
        is_not_the_form = form_name not in " ".join(self.request.steps)
        return (
            is_not_the_form
            and user_is_contributor_in_entity_which_authorize_to_bring_events()
        )


def user_is_contributor_in_entity_which_authorize_to_bring_events():
    user = api.user.get_current() or None
    if user is None:
        return False
    has_permission = False
    brains = api.content.find(object_provides=IEntity.__identifier__)
    for brain in brains:
        entity = brain.getObject()
        if (
            api.user.get_permissions(user=user, obj=entity).get("Modify portal content")
            is True
        ):
            if entity.authorize_to_bring_event_anywhere:
                has_permission = True
                break
    return has_permission



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from plone.api.portal import get_registry_record
from plone.formwidget.geolocation.vocabularies import _ as _geo
from zope.i18n import translate

import json


class UtilsView(BrowserView):
    """ """

    def map_configuration(self):
        """Returns global map configuration from registry"""
        map_layers = get_registry_record("geolocation.map_layers") or []
        config = {
            "fullscreencontrol": get_registry_record("geolocation.fullscreen_control"),
            "locatecontrol": get_registry_record("geolocation.locate_control"),
            "zoomcontrol": get_registry_record("geolocation.zoom_control"),
            "minimap": get_registry_record("geolocation.show_minimap"),
            "addmarker": get_registry_record("geolocation.show_add_marker"),
            "geosearch": get_registry_record("geolocation.show_geosearch"),
            "geosearch_provider": get_registry_record("geolocation.geosearch_provider"),
            "default_map_layer": get_registry_record("geolocation.default_map_layer"),
            "map_layers": [
                {"title": translate(_geo(layer), context=self.request), "id": layer}
                for layer in map_layers
            ],
            "latitude": get_registry_record("geolocation.default_latitude"),
            "longitude": get_registry_record("geolocation.default_longitude"),
        }
        return json.dumps(config)



# -*- coding: utf-8 -*-

from imio.events.core.interfaces import IImioEventsCoreLayer
from imio.events.core.testing import IMIO_EVENTS_CORE_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from zope.component import getMultiAdapter
from zope.interface import alsoProvides

import transaction
import unittest


class TestMultilingual(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            title="Entity",
        )
        self.agenda = api.content.create(
            container=self.entity,
            type="imio.events.Agenda",
            id="imio.events.Agenda",
        )

    def test_create_multilingual_event(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        event = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            title="Mon event que je vais tester en plusieurs langues",
        )
        catalog = api.portal.get_tool("portal_catalog")
        brain = api.content.find(UID=event.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertFalse(indexes.get("translated_in_nl"))
        self.assertFalse(indexes.get("translated_in_de"))
        self.assertFalse(indexes.get("translated_in_en"))

        event.title_en = "My event that I will test in several languages"
        event.reindexObject()
        transaction.commit()
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertFalse(indexes.get("translated_in_nl"))
        self.assertFalse(indexes.get("translated_in_de"))
        self.assertTrue(indexes.get("translated_in_en"))

        event.title_nl = "Mijn evenement die ik in verschillende talen zal testen"
        event.title_de = "Mein Veranstaltung, den ich in mehreren Sprachen testen werde"
        event.reindexObject()
        transaction.commit()
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertTrue(indexes.get("translated_in_nl"))
        self.assertTrue(indexes.get("translated_in_de"))
        self.assertTrue(indexes.get("translated_in_en"))

        event.title_en = None
        event.reindexObject()
        transaction.commit()
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertFalse(indexes.get("translated_in_en"))

    def test_multilingual_searchabletext_event(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        event = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            title="Mon event que je vais tester en plusieurs langues",
        )
        event.title_en = "My event that I will test in several languages"
        event.title_nl = "Mijn evenement die ik in verschillende talen zal testen"
        event.title_de = "Mein Veranstaltung, den ich in mehreren Sprachen testen werde"
        event.description = "Ma description_fr"
        event.description_nl = "Mijn beschrijving"
        event.description_de = "Meine Beschreibung"
        event.description_en = "My description_en"
        event.text = RichTextValue("<p>Mon eventtexte</p>", "text/html", "text/html")
        event.text_en = RichTextValue("<p>My eventtext</p>", "text/html", "text/html")
        event.text_nl = RichTextValue(
            "<p>Mijn eventtekst</p>", "text/html", "text/html"
        )
        event.text_de = RichTextValue(
            "<p>Meine eventetext</p>", "text/html", "text/html"
        )
        event.reindexObject()
        transaction.commit()
        catalog = api.portal.get_tool("portal_catalog")
        brain = api.content.find(UID=event.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertIn("several", indexes.get("SearchableText_en"))
        self.assertIn("verschillende", indexes.get("SearchableText_nl"))
        self.assertIn("mehreren", indexes.get("SearchableText_de"))
        self.assertIn("eventtexte", indexes.get("SearchableText"))
        self.assertIn("eventtext", indexes.get("SearchableText_en"))
        self.assertIn("eventtekst", indexes.get("SearchableText_nl"))
        self.assertIn("eventetext", indexes.get("SearchableText_de"))
        metadatas = catalog.getMetadataForRID(brain.getRID())
        self.assertEqual(event.title_nl, metadatas.get("title_nl"))
        self.assertEqual(event.title_de, metadatas.get("title_de"))
        self.assertEqual(event.title_en, metadatas.get("title_en"))
        self.assertEqual(event.description_nl, metadatas.get("description_nl"))
        self.assertEqual(event.description_de, metadatas.get("description_de"))
        self.assertEqual(event.description_en, metadatas.get("description_en"))

        event.title_en = None
        event.reindexObject()
        transaction.commit()
        catalog = api.portal.get_tool("portal_catalog")
        brain = api.content.find(UID=event.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertNotIn("several", indexes.get("SearchableText_en"))

    def test_event_serializer(self):
        alsoProvides(self.request, IImioEventsCoreLayer)
        event = api.content.create(
            container=self.agenda,
            type="imio.events.Event",
            title="Mon event",
        )
        event.title_en = "My event"
        event.title_nl = "Mijn evenement"
        event.description = "Ma **description**"
        event.description_en = "My **description**"
        event.description_nl = "Mijn **beschrijving**"
        event.text = RichTextValue("<p>Mon texte</p>", "text/html", "text/html")
        event.text_en = RichTextValue("<p>My text</p>", "text/html", "text/html")
        event.text_nl = RichTextValue("<p>Mijn tekst</p>", "text/html", "text/html")

        serializer = getMultiAdapter((event, self.request), ISerializeToJson)
        json = serializer()
        self.assertEqual(json["title"], "Mon event")
        self.assertEqual(json["description"], "Ma **description**")
        self.assertEqual(json["title_fr"], "Mon event")
        self.assertEqual(json["description_fr"], "Ma **description**")

        catalog = api.portal.get_tool("portal_catalog")
        brain = catalog(UID=event.UID())[0]
        serializer = getMultiAdapter((brain, self.request), ISerializeToJsonSummary)
        json_summary = serializer()
        self.assertEqual(json_summary["title"], "Mon event")
        self.assertEqual(json_summary["description"], "Ma description")

        self.request.form["translated_in_nl"] = True
        serializer = getMultiAdapter((event, self.request), ISerializeToJson)
        json = serializer()
        self.assertEqual(json["title"], "Mijn evenement")
        self.assertEqual(json["description"], "Mijn **beschrijving**")
        self.assertEqual(json["text"]["data"], "<p>Mijn tekst</p>")
        self.assertEqual(json["title_fr"], "Mon event")
        self.assertEqual(json["description_fr"], "Ma **description**")
        self.assertEqual(json["text_fr"]["data"], "<p>Mon texte</p>")

        brain = catalog(UID=event.UID())[0]
        serializer = getMultiAdapter((brain, self.request), ISerializeToJsonSummary)
        json_summary = serializer()
        self.assertEqual(json_summary["title"], "Mijn evenement")
        self.assertEqual(json_summary["description"], "Mijn beschrijving")
